#include <Arduino.h>

#include <ArduinoJson.h>
#include <ESP8266WiFi.h>
#include <ESP8266WiFiMulti.h>
#include <ESP8266HTTPClient.h>
#include <WiFiClient.h>


#define PIN        4 
#define NUMPIXELS 19 

const char *ssid = "FSM";
const char *password = "0101010101";
String Shelly_IP = "192.168.178.47";

ESP8266WiFiMulti WiFiMulti;

uint8_t pixels[3*NUMPIXELS];
extern "C" IRAM_ATTR void espShow(uint16_t pin, uint8_t *pixels, uint32_t numBytes, uint8_t type);

const int led = LED_BUILTIN;
int logpwr[9] = {5,10,20,40,80,160,320,640,1280};

int pwrtoled(int l)
{
  int nn;
  if (l < 0)
  {
    nn = -9;
    for (int i=0;i<9;i++) if (l > -logpwr[i]) { nn = -i; break; }
  }
  else if (l >= 0)
  {
    nn = 9;
    for (int i=0;i<9;i++) if (l < logpwr[i]) { nn = i; break; }
  }
  return nn + 9;
}

void setup() 
{
  pinMode(led, OUTPUT);
  digitalWrite(led, LOW);

  Serial.begin(115200);

  pinMode(PIN, OUTPUT);
  for (int i=0;i<3*NUMPIXELS;i++) pixels[i] = 1;
  espShow(PIN,pixels,3*NUMPIXELS,true);
  delay(100);
  
  Serial.println();

  for (uint8_t t = 4; t > 0; t--) 
  {
    Serial.printf("[SETUP] WAIT %d...\n", t);
    Serial.flush();
    delay(1000);
  }
  WiFi.mode(WIFI_STA);
  WiFiMulti.addAP(ssid, password);

  digitalWrite(led, HIGH);
}

int actled;
int pwr1,pwr2,pwr3,pwrsum;

int http_get(String getstr)
{
  StaticJsonDocument<200> doc;
  WiFiClient client;
  HTTPClient http;
  int pwr;

  if (http.begin(client, getstr)) 
  {  
    int httpCode = http.GET();
    if (httpCode > 0) 
    {
      if (httpCode == HTTP_CODE_OK || httpCode == HTTP_CODE_MOVED_PERMANENTLY) 
      {
        String payload = http.getString();
        Serial.println(payload);
        DeserializationError error = deserializeJson(doc, payload);
        if (error) 
        {
          Serial.print(F("deserializeJson() failed: "));
          Serial.println(error.f_str());
        }
        pwr = doc["power"];
      }
    } else {
      Serial.printf("[HTTP] GET... failed, error: %s\n", http.errorToString(httpCode).c_str());
    }
    http.end();
  } else {
    Serial.printf("[HTTP} Unable to connect\n");
  }
  return pwr;
}

void loop() 
{
  // wait for WiFi connection
  if ((WiFiMulti.run() == WL_CONNECTED)) 
  {
    digitalWrite(led, LOW);
    pwr1 = http_get("http://" + Shelly_IP + "/emeter/0");
    pwr2 = http_get("http://" + Shelly_IP + "/emeter/1");
    pwr3 = http_get("http://" + Shelly_IP + "/emeter/2");
    digitalWrite(led, HIGH);
  }
  
  for (int i=0;i<3*NUMPIXELS;i++) pixels[i] = 0;
  pwrsum = pwr1 + pwr2 + pwr3;
  if (pwrsum <= -600) for (int i=0;i<3*NUMPIXELS;i+=3) pixels[i+1] = 1;
  
  actled = pwrtoled(pwr1);
  pixels[3*actled+0] = 50;
  actled = pwrtoled(pwr2);
  pixels[3*actled+1] = 50;
  actled = pwrtoled(pwr3);
  pixels[3*actled+2] = 50;

  espShow(PIN,pixels,3*NUMPIXELS,true);

  delay(5000);
}
